#include "cv.h"
#include "highgui.h"
#include <stdio.h>
#include <ctype.h>
#include <math.h>

int main( int argc, char** argv )
{
    CvCapture* capture = cvCaptureFromCAM(0);

	if(capture == 0)
    {
        fprintf(stderr, "Could not initialize capturing...\n");
        return -1;
    }

    cvNamedWindow("Output");

	IplImage* image = cvCreateImage(cvSize(320, 240), 8, 3);
	IplImage* image2 = cvCreateImage(cvSize(320, 240), 8, 1);
	IplImage* image21 = cvCreateImage(cvSize(320, 240), 8, 1);
	IplImage* image3 = cvCreateImage(cvSize(320, 240), 8, 3);
	IplImage* reference = cvCreateImage(cvSize(320, 240), 8, 3);
	IplConvKernel* kernel = cvCreateStructuringElementEx(5, 5, 2, 2, CV_SHAPE_ELLIPSE);

	double px = 200.0; 
	double py = 100.0;
	double vx = 0.0;
	double vy = 0.0;

    while(true)
    {
        IplImage* frame = cvQueryFrame(capture);

        if(frame == 0)
		{
            break;
		}

		cvFlip(frame, 0, 1);
		cvAbsDiff(frame, reference, image);
		cvSmooth(image, image3, CV_MEDIAN, 5);
		cvInRangeS(image3, cvScalar(10, 10, 10), cvScalar(255, 255, 255), image2);
		cvMorphologyEx(image2, image21, NULL, kernel, CV_MOP_CLOSE, 2); 

		double fx = 0.0;
		double fy = 0.0;

		for(int x = 0; x < image2->width; x++)
		{
			for(int y = 0; y < image2->height; y++)
			{
				if(*(unsigned char*)(image2->imageData + y * image2->widthStep + x) > 128)
				{
					double dx = px - x;
					double dy = py - y;
					double dist = sqrt(dx*dx+dy*dy);
					if(dist < 20)
					{
						double f = 1.0/(dist+0.01);
						fx += dx * f;
						fy += dy * f;
					}
				}
			}
		}
		vx += 0.05 * fx;
		vy += 0.05 * fy;
		px += 0.05 * vx;
		py += 0.05 * vy;

		CvPoint p;
		p.x = (int)px;
		p.y = (int)py;

		cvConvertImage(image21, image);
		cvCircle(image, p, 10, CV_RGB(255, 0, 0), -1);

		// Turn the image to conform to the input
		image->origin = frame->origin;

		cvShowImage("Output", image);

		// cvWaitKey is required to let OpenCV's user interface "breathe".
		int c = cvWaitKey(10);
		if(c == 's')
		{
			cvCopy(frame, reference);
		}
    }

	cvDestroyWindow("Output");
	cvReleaseCapture(&capture);
	cvReleaseImage(&image);
	cvReleaseImage(&image2);
	cvReleaseImage(&image21);
	cvReleaseImage(&image3);
	cvReleaseImage(&reference);
	cvReleaseStructuringElement(&kernel);

    return 0;
}